﻿using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;

namespace XDCSample4
{
    public class ViewModel : ObservableModel
    {
        private ObservableCollection<LayerData> _data;
        private ObservableCollection<LayerData> _vericalDepthValues;
        private Random rnd;
        private int _maxYValue;
        private CustomTickmarkValues _ctv;

        public CustomTickmarkValues CustomTickMarkVals
        {

            get
            {
                return _ctv;
            }
            set
            {
                _ctv = value;
                NotifyPropertyChanged();
            }
        }

        public int MaxYValue
        {
            get
            {
                return _maxYValue;
            }
            set
            {
                _maxYValue = value;
                NotifyPropertyChanged();
            }
        }

        public ObservableCollection<LayerData> Data
        {
            get
            {
                return _data;
            }
            set
            {
                _data = value;
                NotifyPropertyChanged();
            }
        }

        public ObservableCollection<LayerData> VerticalDepthValues
        {
            get
            {
                return _vericalDepthValues;
            }
            set
            {
                _vericalDepthValues = value;
                NotifyPropertyChanged();
            }
        }

        public ViewModel()
        {
            rnd = new Random();
            this.Data = new ObservableCollection<LayerData>() {
                  new LayerData { Layername = "LSS", Value = 7542, Depth = 200, Porosity = 0.3, Sw = 0.5, Temperature = 150  },
            new LayerData { Layername = "LSS base", Value = 7742, Depth = 144, Porosity = 0.25, Sw = 0.15, Temperature = 160 },
            new LayerData { Layername = "Dean", Value = 7886, Depth = 115, Porosity = 0.5, Sw = 0.25, Temperature = 150 },
            new LayerData { Layername = "WA", Value = 8001, Depth = 64, Porosity = 0.45, Sw = 0.15, Temperature = 160 },
            new LayerData { Layername = "WA Middle", Value = 8065, Depth = 65, Porosity = 0.3, Sw = 0.45, Temperature = 150 },
            new LayerData { Layername = "WA Lower", Value = 8130, Depth = 105, Porosity = 0.2, Sw = 0.5, Temperature = 160 }
            };

            this.VerticalDepthValues = this.Data;

            for (int i = 0; i < this.Data.Count; i++)
            {
                if (i == 0)
                {
                    this.VerticalDepthValues[i].Depth = this.Data[i].Depth;

                } else
                {
                    {
                        this.VerticalDepthValues[i].Depth = this.Data[i - 1].Depth + this.Data[i].Depth;
                    }
                }
            }
            this.VerticalDepthValues.Insert(0, new LayerData { Layername = "Dummy initial depth value", Value = 0, Depth =0, Porosity = 0, Sw =0, Temperature = 0 });
            this.MaxYValue = this.VerticalDepthValues.Max(i => i.Depth);
            this.CustomTickMarkVals = new CustomTickmarkValues(this.VerticalDepthValues);
        }
    }

    public class CustomTickmarkValues : TickmarkValues
    {
        public CustomTickmarkValues(ObservableCollection<LayerData> verticalDepthValues)
        {
            // define custom tickmark values
            this.Tickmarks = new DoubleCollection(verticalDepthValues.Select(i => (double)i.Depth).ToArray<double>());
        }
        
        protected double First;
        protected double Last;
        protected DoubleCollection Tickmarks;

        /// <summary>
        /// Initializes tickmark values prior to rendering axis labels, strip bars, and gridlines
        /// </summary>
        /// <param name="initializationParameters"></param>
        public override void Initialize(TickmarkValuesInitializationParameters initializationParameters)
        {
            base.Initialize(initializationParameters);
            // Initialize is overridden in order to store the minimum and maximum values,
            // which will be later used in the MajorValues() and MinorValues() methods.
            this.First = initializationParameters.VisibleMinimum;
            this.Last = initializationParameters.VisibleMaximum;
        }

        /// <summary>
        /// Returns values of major tickmarks
        /// </summary>
        /// <returns></returns>
        public override IEnumerable<double> MajorValues()
        {
            IEnumerable<double> values = Tickmarks.Where((value) => value >= this.First && value <= this.Last);
            return values;
        }

        /// <summary>
        /// Returns values of minor tickmarks
        /// </summary>
        /// <returns></returns>
        public override IEnumerable<double> MinorValues()
        {
            double start = this.First;
            for (double minorValue = start; minorValue < this.Last; minorValue += 1.0)
            {
                yield return minorValue;
            }
        }
    }
}
